[![Build Status](https://img.shields.io/travis/DevelopersToolbox/caretaker/master?style=for-the-badge&logo=travis)](https://travis-ci.com/DevelopersToolbox/caretaker)
[![Software License](https://img.shields.io/badge/license-MIT-blue?style=for-the-badge)](LICENSE.md)
[![Release](https://img.shields.io/github/release/DevelopersToolbox/caretaker?color=blue&style=for-the-badge&logo=github&label=Latest%20Release)](https://github.com/DevelopersToolbox/caretaker/releases/latest)
[![Last Release](https://img.shields.io/github/release-date/DevelopersToolbox/caretaker?color=blue&style=for-the-badge&logo=github)](https://github.com/DevelopersToolbox/caretaker/releases/latest)
[![Github commits (since latest release)](https://img.shields.io/github/commits-since/DevelopersToolbox/caretaker/latest?color=blue&style=for-the-badge&logo=github)](https://github.com/DevelopersToolbox/caretaker/commits)
[![Last Commit](https://img.shields.io/github/last-commit/DevelopersToolbox/caretaker?color=blue&style=for-the-badge&logo=github)](https://github.com/DevelopersToolbox/caretaker/commits/master)
[![Code Size](https://img.shields.io/github/languages/code-size/DevelopersToolbox/caretaker?color=blue&style=for-the-badge&logo=github)](#)
[![Repo Size](https://img.shields.io/github/repo-size/DevelopersToolbox/caretaker?color=blue&style=for-the-badge&logo=github)](#)
[![Contributors](https://img.shields.io/github/contributors/DevelopersToolbox/caretaker?color=blue&style=for-the-badge&logo=github)](https://github.com/DevelopersToolbox/caretaker/graphs/contributors)
[![WolfSoftware](https://img.shields.io/badge/Created%20By-Wolf%20Software%20Ltd.-blue?style=for-the-badge)](https://github.com/WolfSoftware)

<a name="caretaker"></a>
# Caretaker

Caretaker is an automated changelog generator. It works by reading the commit history for the repository and generates the changelog based on that. It requires no human intervention, and will work happily with existing repositories and their existing git histories.

Caretaker also has an advanced category mode, which allows it to parse specially crafted commit messages and split the CHANGELOG into categories (per release).

<a name="additional-features"></a>
## Additional features

* Converting a GitHub username tag into a link to the GitHub user (appended to the commit message).
* Automatically linking each commit message to the hash of that commit.
* Automatically linking each pull request to the pull request number.
* Can run automatically as a post-commit hook.
* Ignore commits that are shorter than a specified number of words.
* Bump verions and tag releases automatically.

> For a full list of featues see the section on command line usage.

<a name="table-of-contents"></a>
## Table of contents

<!--TOCBot-->
* [Caretaker](#caretaker)
     * [Additional features](#additional-features)
     * [Table of contents](#table-of-contents)
     * [Installation](#installation)
     * [Command line usage](#command-line-usage)
          * [Basic usage](#basic-usage)
          * [Initialise the repository](#initialise-the-repository)
               * [The post-commit hook](#the-post-commit-hook)
          * [Customisation](#customisation)
               * [Example of customised usage](#example-of-customised-usage)
               * [Resulting post-commit hook](#resulting-post-commit-hook)
     * [Advanced usage](#advanced-usage)
          * [Category example](#category-example)
               * [Category list](#category-list)
               * [Example category output](#example-category-output)
          * [Tagging release versions](#tagging-release-versions)
               * [Release handler snippet](#release-handler-snippet)
               * [Bumping release versions](#bumping-release-versions)
          * [Caretaker config file](#caretaker-config-file)
               * [Example config file](#example-config-file)
          * [Github usernames](#github-usernames)
               * [Github username example](#github-username-example)
               * [Github username command line example](#github-username-command-line-example)
               * [Result](#result)
          * [Pull requests and commits](#pull-requests-and-commits)
               * [Pull request linking example](#pull-request-linking-example)
          * [Issues](#issues)
               * [Issues link example](#issues-link-example)
               * [Issues link result](#issues-link-result)
     * [Using caretaker in your own code](#using-caretaker-in-your-own-code)
          * [Installation](#installation)
          * [Usage](#usage)
               * [Load the gem](#load-the-gem)
               * [Initialise](#initialise)
               * [Generate a changelog](#generate-a-changelog)
               * [Initialise the git repo](#initialise-the-git-repo)
     * [Contributing to caretaker](#contributing-to-caretaker)
          * [Setup](#setup)
          * [Testing](#testing)
* [To-Do List](#to-do-list)
<!--TOCBot-->

<a name="installation"></a>
## Installation

Installing caretaker is very simple, just execute the following command.

```
gem install caretaker
```

<a name="command-line-usage"></a>
## Command line usage

Caretaker has a number of command line options:

```
Usage: caretaker [-h] [-a author] [-c] [-e] [-i] [-o filename] [-r] [-s] [-u] [-w number]
```

| Option | Purpose | Default Value |
| ------ | ------- |:-------------:|
| -h or --help | Show the help message | N/A |
| -a or --author [Github username] | Specify the author name | |
| -b or --bump [part] | Bump the VERSION.txt to next version. | |
| -c or --config | Generate a .caretaker.cfg config file. | false |
| -e or --enable-categories | Enable the splitting of commit messages into categories | false |
| -i or --init | Initialise the repo to use caretaker | |
| -o or --output [filename] | Set the name of the output file | CHANGELOG.md |
| -r or --remove-categories | Remove categories from commit messages. --enable-categories sets this to true | false |
| -s or --silent | Turn off all output from Caretaker, aka Silent Mode | false |
| -u or --url-verification | Verify each url to ensure that the links are valid, skip any links that are not | false |
| -w or --words number | Minimum number of words needed to include a commit. | 1 |

<a name="basic-usage"></a>
### Basic usage

<a name="initialise-the-repository"></a>
### Initialise the repository

This step isn't required but running the following command configures your repository to use caretaker automatically on every commit to ensure that your changelog is as up-to-date as possible.

```
caretaker -i
```

What is actually does is create a post-commit hook in your git repo which execute caretaker and adds the updated CHANGELOG.md to the commit ready for you to push.

<a name="the-post-commit-hook"></a>
#### The post-commit hook

```
#!/usr/bin/env bash

OUTPUT_FILE="CHANGELOG.md"

caretaker

res=$(git status --porcelain | grep "${OUTPUT_FILE}" | wc -l)
if [[ "${res}" -gt 0 ]]; then
    git add "${OUTPUT_FILE}"
    git commit --amend --no-edit
end
```

<a name="customisation"></a>
### Customisation

There are a number of ways in which you can customise the way caretaker works, you can enable category mode, or change the output file from CHANGELOG.md to some other name. If you do this as part of the initialisation then the settings are retained.

<a name="example-of-customised-usage"></a>
#### Example of customised usage
```
caretaker -i -e -o HISTORY.md
```

<a name="resulting-post-commit-hook"></a>
#### Resulting post-commit hook

```
#!/usr/bin/env bash

OUTPUT_FILE="CHANGELOG.md"

caretaker -i -e -o HISTORY.md

res=$(git status --porcelain | grep "${OUTPUT_FILE}" | wc -l)
if [[ "${res}" -gt 0 ]]; then
    git add "${OUTPUT_FILE}"
    git commit --amend --no-edit
end
```

It is worth noting the final git commit command the ```--amend``` tells git to amend the last commit and add in the output file and the ```--no-edit``` tells git that we do not want to edit the commit message and to retain the previous one.

Once the initialisation is complete, you should never need to do anything else as caretaker will run automatically and keep your changelog up-to-date. You can re-run the init command at anytime if you wish to change the configuration that you are using.

If you do not want to run caretaker automatically, maybe you are just evaluating it and are not yet ready to fully automate the process, then you can run caretaker manually from the command line.

<a name="advanced-usage"></a>
## Advanced usage

Amongst other things Caretaker has a category mode (-e or --enable-categories) which allows for the grouping of changes by category. This is done by using special tags as a prefix to your commit messaged. It will parse these prefixes and group like commits together, within a specific release.

```
git commit -m "<category>: <message>"
```

<a name="category-example"></a>
### Category example

```
git commit -m "docs: Updated the README.md to include an example."
```

<a name="category-list"></a>
#### Category list

The categories are used for splitting the changelog into logical sections. In the following table the category is what is displayed in the CHANGELOG.md (as the section header), and the patterns are the strings that it will match on when parsing the commit messages.

> The matching is case-insensitive.


| Category | Patterns | Purpose |
| -------- | -------- | ------- |
| New Features | new features, new feature, new, feature | Any new features that have been added |
| Improvements | improvements, improvement | Any improvement or enhancement of an existing feature |
| Bug Fixes | bug fixes, bug fix, bug, bugs | Any bug fixes to existing features |
| Security Fixes | security fixes, security | Any security fixes to existing features |
| Refactor | refactor | A code change that neither fixes a bug or adds a feature |
| Style | style | A change that does not affect the meaning of the code (white-space, formatting, missing semi-colons, etc) |
| Deprecated | deprecated | Any existing feature that is being deprecated |
| Removed | removed | Any existing feature that is being removed |
| Tests | tests, test, testing | Any changes to the existing tests |
| Documentation | documentation, docs | Changes to documentation only |
| Chores | chores, chore | Changes to the build process or auxiliary tools and libraries such as documentation generation |
| Experiments | experiments, experiment | Any experiments that you are carrying out, these are things that might get added and then removed again |
| Miscellaneous | miscellaneous, misc | Anything else (this shouldn't really be required) |
| Initial Commit | initial commit, initial | A special tag for the initial commit to the repo |
| Skip | skip, ignore | A special tag to ignore adding the commit to the CHANGELOG |

> Any commit without a matching category will be added to an 'Uncategorised' group.

<a name="example-category-output"></a>
#### Example category output

```
# Changelog

All notable changes to this project will be documented in this file.

### [Unreleased](https://github.com/DevelopersToolbox/caretaker/compare/v1.0.0...HEAD)

###### Documentation:

- Updated the README.md to include an example.

###### Uncategorised:

- Created an example without a prefix.
```

<a name="tagging-release-versions"></a>
### Tagging release versions

Due to the way that post-commit hooks work, if you were tag a release that tag would either be ignored or worse lost, because the commit ID changes when a 'commit amend' is carried out. In order to get around this problem Caretaker can now tag releases
for you as part of its push process.

The current version is stored in 'VERSION.txt', and it is semantic version string (however it can include an optional v prefix eg v1.0.0). This version is used to 'tag' a release as part of the process. It works be comparing the current tag to the
one in the file and if they are different it will tag the release with the new version.

<a name="release-handler-snippet"></a>
#### Release handler snippet

The following code is a snippet of how the release mechanism works:

```
if [[ -n $RELEASE_VERSION ]]; then
    git tag "${TAG_VERSION}"
fi

#{cmd}

res=$(git status --porcelain | grep "${OUTPUT_FILE}" | wc -l)
if [[ "${res}" -gt 0 ]]; then
    git add "${OUTPUT_FILE}" >> /dev/null 2>&1
    git commit --amend --no-edit >> /dev/null 2>&1

    if [[ -n "${RELEASE_VERSION"} ]]; then
        git tag -f "${TAG_VERSION}"
    fi
fi
```

As you can see, it will tag the release, run caretaker and re-tag the release after the commit amend, ensuring that the tag persists. This is not a perfect solution, but it does work :)

> The created tag is *NOT* pushed to the remote end, this is to allow manual pushing once the release is correct.

<a name="bumping-release-versions"></a>
#### Bumping release versions

In order to make life as simple as possible the new version of Caretaker comes with a ```--bump``` option, this takes 1 argument of either ```major```, ```minor``` or ```patch``` and will update the version file accordingly.

> Note if the version file doesn't exist, it will be created with a default version of v0.1.0

<a name="caretaker-config-file"></a>
### Caretaker config file

Caretaker allows you to generate a config file (-c or --config-file) which it will read automatically, so that you do not need to provide command line flags each time you run Caretaker, this file is also read when you run Caretaker via the post-commit hook.

It will look in the base of the current git repository for the config file, if it is not found it will then check your home directory, if still not found it will check the current directory and every parent directory until it either finds a config file or runs out of directories to check. Placing the config file, for example, in your home directory will have the effect of making it global for ALL your git repositories.

> The config file is called ```.caretaker.yml```.

You can run Caretaker with command line options and if you add the -c option it will generate the config (in the base of your git repository) with the settings you specificy.

<a name="example-config-file"></a>
#### Example config file

```
enable-categories: false
min-words: 1
remove-categories: false
silent: false
verify-urls: false
```

<a name="github-usernames"></a>
### Github usernames

Caretaker is also able to parse commit messages for special tags which specify a GitHub username and converts those tags into a link to the user. This link appended to the commit messages.

<a name="github-username-example"></a>
#### Github username example
```
git commit -m "docs: This is a GitHub username example. {TGWolf}"
```

In addition you can set a username using the -a option:

<a name="github-username-command-line-example"></a>
#### Github username command line example
```
git commit -m "docs: This is a GitHub username example." -a TGWolf
```

You can also use the -a option along with the -i (init) option to set the username as your default for all commits.

If there is already a {author} tag within the commit then the -a option is ignored for that commit, it is only appended to commits with no author. This stops double entries if you enable -a after you have already tagged commits with an author.

<a name="result"></a>
#### Result

```
- This is a GitHub username example. [`[Author: TGWolf]`](https://github.com/TGWolf)
```

<a name="pull-requests-and-commits"></a>
### Pull requests and commits

Caretaker is also able to identify the difference between pull requests and commits and automatically creates a link to either the pull request number or the commit it.

<a name="pull-request-linking-example"></a>
#### Pull request linking example

```
- This is a normal commit. [`[17df108]`](https://github.com/DevelopersToolbox/caretaker/commit/17df1080a501ed1bb780aa946190d64966dc30df)
- This is a pull request. [`[#7]`](https://github.com/DevelopersToolbox/caretaker/pull/7)
```

<a name="issues"></a>
### Issues

It is also possible to specify a tag within the commit to tell caretaker that the commit relates to an issue and caretaker will create a link to that issues.

<a name="issues-link-example"></a>
#### Issues link example

```
git commit -m "Bug: This fixes a bug in the code and resolves issue (issue-7).
```

<a name="issues-link-result"></a>
#### Issues link result

```
##### Bug Fixes:

This fixes a bug in the code and resolves issue [`[#7]`](https://github.com/DevelopersToolbox/caretaker/issues/7)
```

<a name="using-caretaker-in-your-own-code"></a>
## Using caretaker in your own code

<a name="installation"></a>
### Installation

Add this line to your application's Gemfile:

```ruby
gem 'caretaker'
```

And then execute:

```
$ bundle install
```

Or install it yourself as:

```
$ gem install caretaker
```

<a name="usage"></a>
### Usage

<a name="load-the-gem"></a>
#### Load the gem
```ruby
require 'caretaker'
```

<a name="initialise"></a>
#### Initialise

```ruby
caretaker = Caretaker.new(output: 'HISTORY.md')
```
> output is `optional` and will default to `CHANGELOG.md`

<a name="generate-a-changelog"></a>
#### Generate a changelog

```ruby
Caretaker.generate_changelog
```

<a name="initialise-the-git-repo"></a>
#### Initialise the git repo

```Ruby
Caretaker.init_repo
```

<a name="contributing-to-caretaker"></a>
## Contributing to caretaker

<a name="setup"></a>
### Setup

After checking out the repo, run `bin/setup` to install dependencies. Then, run `rake spec` to run the tests. You can also run `bin/console` for an interactive prompt that will allow you to experiment.

To install this gem onto your local machine, run `bundle exec rake install`. To release a new version, update the version number in `version.rb`, and then run `bundle exec rake release`, which will
create a git tag for the version, push git commits and tags, and push the `.gem` file to [rubygems.org](https://rubygems.org).

<a name="testing"></a>
### Testing

For local testing make sure that you run `bundle exec rspec spec` and then `rake install` to install the gem locally.

For further information please refer to the [contributing](https://github.com/WolfSoftware/contributing) documentation.

<a name="to-do-list"></a>
# To-Do List

- [ ] CLI option to enable/disable linking to commit hashes
- [ ] CLI option to enable/disable linking to pull requests
- [ ] CLI option to enable/disable linking to github users
- [ ] Option to allow auto-pushing of the tag to the remote origin
